/*
Configuration page for the print journal mode.
*/

/********************************************************************************
** Form generated from reading UI file 'calprintjournalconfig_base.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CALPRINTJOURNALCONFIG_BASE_H
#define UI_CALPRINTJOURNALCONFIG_BASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kdatecombobox.h"

QT_BEGIN_NAMESPACE

class Ui_CalPrintJournalConfig_Base
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *label;
    QGroupBox *mDateRangeGroup;
    QVBoxLayout *vboxLayout1;
    QRadioButton *mAllJournals;
    QRadioButton *mRangeJournals;
    QHBoxLayout *mDateRangeLayout;
    QSpacerItem *spacerItem;
    QLabel *mFromDateLabel;
    KDateComboBox *mFromDate;
    QLabel *mToDateLabel;
    KDateComboBox *mToDate;
    QSpacerItem *spacerItem1;
    QGroupBox *mSecurity;
    QHBoxLayout *horizontalLayout;
    QCheckBox *mExcludeConfidential;
    QCheckBox *mExcludePrivate;
    QGroupBox *mGeneralGroup;
    QVBoxLayout *verticalLayout;
    QCheckBox *mPrintFooter;
    QSpacerItem *spacerItem2;
    QButtonGroup *buttonGroup;

    void setupUi(QWidget *CalPrintJournalConfig_Base)
    {
        if (CalPrintJournalConfig_Base->objectName().isEmpty())
            CalPrintJournalConfig_Base->setObjectName("CalPrintJournalConfig_Base");
        vboxLayout = new QVBoxLayout(CalPrintJournalConfig_Base);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(-1, -1, 0, -1);
        label = new QLabel(CalPrintJournalConfig_Base);
        label->setObjectName("label");

        vboxLayout->addWidget(label);

        mDateRangeGroup = new QGroupBox(CalPrintJournalConfig_Base);
        mDateRangeGroup->setObjectName("mDateRangeGroup");
        vboxLayout1 = new QVBoxLayout(mDateRangeGroup);
        vboxLayout1->setObjectName("vboxLayout1");
        mAllJournals = new QRadioButton(mDateRangeGroup);
        buttonGroup = new QButtonGroup(CalPrintJournalConfig_Base);
        buttonGroup->setObjectName("buttonGroup");
        buttonGroup->addButton(mAllJournals);
        mAllJournals->setObjectName("mAllJournals");

        vboxLayout1->addWidget(mAllJournals);

        mRangeJournals = new QRadioButton(mDateRangeGroup);
        buttonGroup->addButton(mRangeJournals);
        mRangeJournals->setObjectName("mRangeJournals");
        mRangeJournals->setFocusPolicy(Qt::TabFocus);

        vboxLayout1->addWidget(mRangeJournals);

        mDateRangeLayout = new QHBoxLayout();
        mDateRangeLayout->setObjectName("mDateRangeLayout");
        spacerItem = new QSpacerItem(24, 1, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        mDateRangeLayout->addItem(spacerItem);

        mFromDateLabel = new QLabel(mDateRangeGroup);
        mFromDateLabel->setObjectName("mFromDateLabel");

        mDateRangeLayout->addWidget(mFromDateLabel);

        mFromDate = new KDateComboBox(mDateRangeGroup);
        mFromDate->setObjectName("mFromDate");
        mFromDate->setFocusPolicy(Qt::StrongFocus);

        mDateRangeLayout->addWidget(mFromDate);

        mToDateLabel = new QLabel(mDateRangeGroup);
        mToDateLabel->setObjectName("mToDateLabel");

        mDateRangeLayout->addWidget(mToDateLabel);

        mToDate = new KDateComboBox(mDateRangeGroup);
        mToDate->setObjectName("mToDate");
        mToDate->setFocusPolicy(Qt::StrongFocus);

        mDateRangeLayout->addWidget(mToDate);

        spacerItem1 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        mDateRangeLayout->addItem(spacerItem1);


        vboxLayout1->addLayout(mDateRangeLayout);


        vboxLayout->addWidget(mDateRangeGroup);

        mSecurity = new QGroupBox(CalPrintJournalConfig_Base);
        mSecurity->setObjectName("mSecurity");
        horizontalLayout = new QHBoxLayout(mSecurity);
        horizontalLayout->setObjectName("horizontalLayout");
        mExcludeConfidential = new QCheckBox(mSecurity);
        mExcludeConfidential->setObjectName("mExcludeConfidential");

        horizontalLayout->addWidget(mExcludeConfidential);

        mExcludePrivate = new QCheckBox(mSecurity);
        mExcludePrivate->setObjectName("mExcludePrivate");

        horizontalLayout->addWidget(mExcludePrivate);


        vboxLayout->addWidget(mSecurity);

        mGeneralGroup = new QGroupBox(CalPrintJournalConfig_Base);
        mGeneralGroup->setObjectName("mGeneralGroup");
        verticalLayout = new QVBoxLayout(mGeneralGroup);
        verticalLayout->setObjectName("verticalLayout");
        mPrintFooter = new QCheckBox(mGeneralGroup);
        mPrintFooter->setObjectName("mPrintFooter");

        verticalLayout->addWidget(mPrintFooter);


        vboxLayout->addWidget(mGeneralGroup);

        spacerItem2 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem2);

#if QT_CONFIG(shortcut)
        mFromDateLabel->setBuddy(mFromDate);
        mToDateLabel->setBuddy(mToDate);
#endif // QT_CONFIG(shortcut)

        retranslateUi(CalPrintJournalConfig_Base);
        QObject::connect(mRangeJournals, SIGNAL(toggled(bool)), mFromDate, SLOT(setEnabled(bool)));
        QObject::connect(mRangeJournals, &QRadioButton::toggled, mFromDateLabel, &QLabel::setEnabled);
        QObject::connect(mRangeJournals, SIGNAL(toggled(bool)), mToDate, SLOT(setEnabled(bool)));
        QObject::connect(mRangeJournals, &QRadioButton::toggled, mToDateLabel, &QLabel::setEnabled);

        QMetaObject::connectSlotsByName(CalPrintJournalConfig_Base);
    } // setupUi

    void retranslateUi(QWidget *CalPrintJournalConfig_Base)
    {
        label->setText(tr2i18n("<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Print journal options:</span></p></body></html>", nullptr));
        mDateRangeGroup->setTitle(tr2i18n("Date && Time Range", nullptr));
        mAllJournals->setText(tr2i18n("&All journal entries", nullptr));
        mRangeJournals->setText(tr2i18n("Date &range:", nullptr));
#if QT_CONFIG(tooltip)
        mFromDateLabel->setToolTip(tr2i18n("Starting date for the print", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mFromDateLabel->setWhatsThis(tr2i18n("If you want to print more days at once, you can define a range of dates with this option and the <i>End date</i> option. This option is used to define the start date.", nullptr));
#endif // QT_CONFIG(whatsthis)
        mFromDateLabel->setText(tr2i18n("&Start date:", nullptr));
#if QT_CONFIG(tooltip)
        mFromDate->setToolTip(tr2i18n("Starting date for the print", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mFromDate->setWhatsThis(tr2i18n("If you want to print more days at once, you can define a range of dates with this option and the <i>End date</i> option. This option is used to define the start date.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        mToDateLabel->setToolTip(tr2i18n("Ending date for the print", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mToDateLabel->setWhatsThis(tr2i18n("If you want to print more days at once, you can define a range of dates with this option and the <i>Start date</i> option. This option is used to define the end date.", nullptr));
#endif // QT_CONFIG(whatsthis)
        mToDateLabel->setText(tr2i18n("&End date:", nullptr));
#if QT_CONFIG(tooltip)
        mToDate->setToolTip(tr2i18n("Ending date for the print", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mToDate->setWhatsThis(tr2i18n("If you want to print more days at once, you can define a range of dates with this option and the <i>Start date</i> option. This option is used to define the end date.", nullptr));
#endif // QT_CONFIG(whatsthis)
        mSecurity->setTitle(tr2i18n("Security Exclusions", nullptr));
#if QT_CONFIG(tooltip)
        mExcludeConfidential->setToolTip(tr2i18n("Check this option to exclude items that have their Access level set to \342\200\234Confidential\342\200\235", nullptr));
#endif // QT_CONFIG(tooltip)
        mExcludeConfidential->setText(tr2i18n("Exclude c&onfidential", nullptr));
#if QT_CONFIG(tooltip)
        mExcludePrivate->setToolTip(tr2i18n("Check this option to exclude items that have their Access level set to \342\200\234Private\342\200\235", nullptr));
#endif // QT_CONFIG(tooltip)
        mExcludePrivate->setText(tr2i18n("Exclude pri&vate", nullptr));
        mGeneralGroup->setTitle(tr2i18n("General", "@title general print settings"));
#if QT_CONFIG(tooltip)
        mPrintFooter->setToolTip(tr2i18n("Print a datetime footer on each page", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mPrintFooter->setWhatsThis(tr2i18n("Check this box if you want to print a small footer on each page that contains the date of the print.", nullptr));
#endif // QT_CONFIG(whatsthis)
        mPrintFooter->setText(tr2i18n("Print &Footer", nullptr));
        (void)CalPrintJournalConfig_Base;
    } // retranslateUi

};

namespace Ui {
    class CalPrintJournalConfig_Base: public Ui_CalPrintJournalConfig_Base {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CALPRINTJOURNALCONFIG_BASE_H

