/*
Configuration page for the yearly print mode.
*/

/********************************************************************************
** Form generated from reading UI file 'calprintyearconfig_base.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CALPRINTYEARCONFIG_BASE_H
#define UI_CALPRINTYEARCONFIG_BASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_CalPrintYearConfig_Base
{
public:
    QVBoxLayout *verticalLayout_3;
    QLabel *label;
    QGroupBox *mDateRangeGroup;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QLabel *mYearLabel;
    QSpinBox *mYear;
    QSpacerItem *spacer4;
    QGroupBox *mSecurity;
    QHBoxLayout *horizontalLayout_4;
    QCheckBox *mExcludeConfidential;
    QCheckBox *mExcludePrivate;
    QGroupBox *mGeneralGroup;
    QGridLayout *gridLayout;
    QLabel *mPagesLabel;
    QComboBox *mPages;
    QLabel *mSubDaysLabel;
    QComboBox *mSubDays;
    QLabel *mHolidaysLabel;
    QComboBox *mHolidays;
    QCheckBox *mPrintFooter;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *CalPrintYearConfig_Base)
    {
        if (CalPrintYearConfig_Base->objectName().isEmpty())
            CalPrintYearConfig_Base->setObjectName("CalPrintYearConfig_Base");
        verticalLayout_3 = new QVBoxLayout(CalPrintYearConfig_Base);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(-1, -1, 0, -1);
        label = new QLabel(CalPrintYearConfig_Base);
        label->setObjectName("label");

        verticalLayout_3->addWidget(label);

        mDateRangeGroup = new QGroupBox(CalPrintYearConfig_Base);
        mDateRangeGroup->setObjectName("mDateRangeGroup");
        mDateRangeGroup->setProperty("selectedId", QVariant(-1));
        verticalLayout_2 = new QVBoxLayout(mDateRangeGroup);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        mYearLabel = new QLabel(mDateRangeGroup);
        mYearLabel->setObjectName("mYearLabel");

        horizontalLayout->addWidget(mYearLabel);

        mYear = new QSpinBox(mDateRangeGroup);
        mYear->setObjectName("mYear");
        mYear->setMinimum(0);
        mYear->setMaximum(2500);
        mYear->setValue(2012);

        horizontalLayout->addWidget(mYear);

        spacer4 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(spacer4);


        verticalLayout_2->addLayout(horizontalLayout);


        verticalLayout_3->addWidget(mDateRangeGroup);

        mSecurity = new QGroupBox(CalPrintYearConfig_Base);
        mSecurity->setObjectName("mSecurity");
        horizontalLayout_4 = new QHBoxLayout(mSecurity);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        mExcludeConfidential = new QCheckBox(mSecurity);
        mExcludeConfidential->setObjectName("mExcludeConfidential");

        horizontalLayout_4->addWidget(mExcludeConfidential);

        mExcludePrivate = new QCheckBox(mSecurity);
        mExcludePrivate->setObjectName("mExcludePrivate");

        horizontalLayout_4->addWidget(mExcludePrivate);


        verticalLayout_3->addWidget(mSecurity);

        mGeneralGroup = new QGroupBox(CalPrintYearConfig_Base);
        mGeneralGroup->setObjectName("mGeneralGroup");
        gridLayout = new QGridLayout(mGeneralGroup);
        gridLayout->setObjectName("gridLayout");
        mPagesLabel = new QLabel(mGeneralGroup);
        mPagesLabel->setObjectName("mPagesLabel");
        mPagesLabel->setWordWrap(false);

        gridLayout->addWidget(mPagesLabel, 0, 0, 1, 1);

        mPages = new QComboBox(mGeneralGroup);
        mPages->setObjectName("mPages");

        gridLayout->addWidget(mPages, 0, 1, 1, 1);

        mSubDaysLabel = new QLabel(mGeneralGroup);
        mSubDaysLabel->setObjectName("mSubDaysLabel");
        mSubDaysLabel->setWordWrap(false);

        gridLayout->addWidget(mSubDaysLabel, 1, 0, 1, 1);

        mSubDays = new QComboBox(mGeneralGroup);
        mSubDays->addItem(QString());
        mSubDays->addItem(QString());
        mSubDays->setObjectName("mSubDays");

        gridLayout->addWidget(mSubDays, 1, 1, 1, 1);

        mHolidaysLabel = new QLabel(mGeneralGroup);
        mHolidaysLabel->setObjectName("mHolidaysLabel");
        mHolidaysLabel->setWordWrap(false);

        gridLayout->addWidget(mHolidaysLabel, 2, 0, 1, 1);

        mHolidays = new QComboBox(mGeneralGroup);
        mHolidays->addItem(QString());
        mHolidays->addItem(QString());
        mHolidays->setObjectName("mHolidays");

        gridLayout->addWidget(mHolidays, 2, 1, 1, 1);

        mPrintFooter = new QCheckBox(mGeneralGroup);
        mPrintFooter->setObjectName("mPrintFooter");

        gridLayout->addWidget(mPrintFooter, 3, 0, 1, 2);


        verticalLayout_3->addWidget(mGeneralGroup);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        mYearLabel->setBuddy(mYear);
        mPagesLabel->setBuddy(mPages);
        mSubDaysLabel->setBuddy(mSubDays);
        mHolidaysLabel->setBuddy(mHolidays);
#endif // QT_CONFIG(shortcut)

        retranslateUi(CalPrintYearConfig_Base);

        mSubDays->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(CalPrintYearConfig_Base);
    } // setupUi

    void retranslateUi(QWidget *CalPrintYearConfig_Base)
    {
        label->setText(tr2i18n("<html><head/><body><p><span style=\" font-size:9pt; font-weight:600;\">Print year options:</span></p></body></html>", nullptr));
        mDateRangeGroup->setTitle(tr2i18n("Date Range", nullptr));
        mYearLabel->setText(tr2i18n("&Year:", nullptr));
        mSecurity->setTitle(tr2i18n("Security Exclusions", nullptr));
#if QT_CONFIG(tooltip)
        mExcludeConfidential->setToolTip(tr2i18n("Check this option to exclude items that have their Access level set to \342\200\234Confidential\342\200\235", nullptr));
#endif // QT_CONFIG(tooltip)
        mExcludeConfidential->setText(tr2i18n("Exclude c&onfidential", nullptr));
#if QT_CONFIG(tooltip)
        mExcludePrivate->setToolTip(tr2i18n("Check this option to exclude items that have their Access level set to \342\200\234Private\342\200\235", nullptr));
#endif // QT_CONFIG(tooltip)
        mExcludePrivate->setText(tr2i18n("Exclude pri&vate", nullptr));
        mGeneralGroup->setTitle(tr2i18n("General", nullptr));
        mPagesLabel->setText(tr2i18n("Num&ber of pages:", nullptr));
        mSubDaysLabel->setText(tr2i18n("Show &sub-day events as:", nullptr));
        mSubDays->setItemText(0, tr2i18n("Text", nullptr));
        mSubDays->setItemText(1, tr2i18n("Time Boxes", nullptr));

        mHolidaysLabel->setText(tr2i18n("Show &holidays as:", nullptr));
        mHolidays->setItemText(0, tr2i18n("Text", nullptr));
        mHolidays->setItemText(1, tr2i18n("Time Boxes", nullptr));

#if QT_CONFIG(tooltip)
        mPrintFooter->setToolTip(tr2i18n("Print a datetime footer on each page", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        mPrintFooter->setWhatsThis(tr2i18n("Check this box if you want to print a small footer on each page that contains the date of the print.", nullptr));
#endif // QT_CONFIG(whatsthis)
        mPrintFooter->setText(tr2i18n("Print &Footer", nullptr));
        (void)CalPrintYearConfig_Base;
    } // retranslateUi

};

namespace Ui {
    class CalPrintYearConfig_Base: public Ui_CalPrintYearConfig_Base {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CALPRINTYEARCONFIG_BASE_H

