// This file is generated by kconfig_compiler_kf6 from kcalprefs.kcfg.
// All changes you do to this file will be lost.

#include "kcalprefs_base.h"

#include <klocalizedstring.h>

using namespace CalendarSupport;

KCalPrefsBase::KCalPrefsBase( )
  : KConfigSkeleton( QStringLiteral( "korganizerrc" ) )
{
  setCurrentGroup( QStringLiteral( "Personal Settings" ) );

  mEmailControlCenterItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Use Control Center Email" ), mEmailControlCenter, true );
  mEmailControlCenterItem->setLabel( i18nd("calendarsupport6", "Use email settings from System Settings") );
  mEmailControlCenterItem->setWhatsThis( i18nd("calendarsupport6", "Check this box to use the KDE-wide e-mail settings, which are defined in the “Users” module in System Settings. Uncheck this box to be able to specify your full name and e-mail.") );
  addItem( mEmailControlCenterItem, QStringLiteral( "EmailControlCenter" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesEventSecrecyPolicy;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("EventSecrecyPublic");
    choice.label = i18nd("calendarsupport6", "Public (accessible to everyone)");
    valuesEventSecrecyPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("EventSecrecyConfidential");
    choice.label = i18nd("calendarsupport6", "Confidential (accessible to you and others)");
    valuesEventSecrecyPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("EventSecrecyPrivate");
    choice.label = i18nd("calendarsupport6", "Private (accessible to you only)");
    valuesEventSecrecyPolicy.append( choice );
  }
  mEventSecrecyPolicyItem = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "Event Secrecy Policy" ), mEventSecrecyPolicy, valuesEventSecrecyPolicy, EventSecrecyPublic );
  mEventSecrecyPolicyItem->setLabel( i18nd("calendarsupport6", "Events:") );
  addItem( mEventSecrecyPolicyItem, QStringLiteral( "EventSecrecyPolicy" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesTodoSecrecyPolicy;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("TodoSecrecyPublic");
    choice.label = i18nd("calendarsupport6", "Public (accessible to everyone)");
    valuesTodoSecrecyPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("TodoSecrecyConfidential");
    choice.label = i18nd("calendarsupport6", "Confidential (accessible to you and some others)");
    valuesTodoSecrecyPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("TodoSecrecyPrivate");
    choice.label = i18nd("calendarsupport6", "Private (accessible to you only)");
    valuesTodoSecrecyPolicy.append( choice );
  }
  mTodoSecrecyPolicyItem = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "Todo Secrecy Policy" ), mTodoSecrecyPolicy, valuesTodoSecrecyPolicy, TodoSecrecyPublic );
  mTodoSecrecyPolicyItem->setLabel( i18nd("calendarsupport6", "To-dos:") );
  addItem( mTodoSecrecyPolicyItem, QStringLiteral( "TodoSecrecyPolicy" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesJournalSecrecyPolicy;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("JournalSecrecyPublic");
    choice.label = i18nd("calendarsupport6", "Public (accessible to everyone)");
    valuesJournalSecrecyPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("JournalSecrecyConfidential");
    choice.label = i18nd("calendarsupport6", "Confidential (accessible to you and some others)");
    valuesJournalSecrecyPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("JournalSecrecyPrivate");
    choice.label = i18nd("calendarsupport6", "Private (accessible to you only)");
    valuesJournalSecrecyPolicy.append( choice );
  }
  mJournalSecrecyPolicyItem = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "Journal Secrecy Policy" ), mJournalSecrecyPolicy, valuesJournalSecrecyPolicy, JournalSecrecyPrivate );
  mJournalSecrecyPolicyItem->setLabel( i18nd("calendarsupport6", "Journals:") );
  addItem( mJournalSecrecyPolicyItem, QStringLiteral( "JournalSecrecyPolicy" ) );

  setCurrentGroup( QStringLiteral( "Default Datetimes" ) );

  mStartTimeItem = new KConfigSkeleton::ItemDateTime( currentGroup(), QStringLiteral( "StartTime" ), mStartTime, QDateTime(QDate(1752,1,1), QTime(10,0)) );
  mStartTimeItem->setLabel( i18nd("calendarsupport6", "Default appointment time") );
  mStartTimeItem->setWhatsThis( i18nd("calendarsupport6", "Enter the default time for events here. The default is used if you do not supply a start time.") );
  addItem( mStartTimeItem, QStringLiteral( "StartTime" ) );
  mDefaultDurationItem = new KConfigSkeleton::ItemDateTime( currentGroup(), QStringLiteral( "DefaultDuration" ), mDefaultDuration, QDateTime(QDate(1752,1,1), QTime(1,0)) );
  mDefaultDurationItem->setLabel( i18nd("calendarsupport6", "Default duration of new appointment (HH:MM)") );
  mDefaultDurationItem->setWhatsThis( i18nd("calendarsupport6", "Enter default duration for events here. The default is used if you do not supply an end time.") );
  addItem( mDefaultDurationItem, QStringLiteral( "DefaultDuration" ) );
  mDefaultAudioFileRemindersItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Enable Default Audio File" ), mDefaultAudioFileReminders, false );
  mDefaultAudioFileRemindersItem->setLabel( i18nd("calendarsupport6", "Enable a default sound file for audio reminders") );
  mDefaultAudioFileRemindersItem->setWhatsThis( i18nd("calendarsupport6", "Check this box if you want to enable the specified file to be used as the default sound file for new reminders. You can always specify another file in the Reminder configuration accessible from the Event or To-do editors.") );
  addItem( mDefaultAudioFileRemindersItem, QStringLiteral( "defaultAudioFileReminders" ) );
  mAudioFilePathItem = new KConfigSkeleton::ItemPath( currentGroup(), QStringLiteral( "Reminder Audio File" ), mAudioFilePath );
  mAudioFilePathItem->setLabel( i18nd("calendarsupport6", "Default audio file") );
  mAudioFilePathItem->setWhatsThis( i18nd("calendarsupport6", "Set a file to be used as the default sound file for new reminders. You can always specify another file in the Reminder configuration accessible from the Event or To-do editors.") );
  addItem( mAudioFilePathItem, QStringLiteral( "audioFilePath" ) );
  mDefaultEventRemindersItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Enable Event Reminders" ), mDefaultEventReminders, true );
  mDefaultEventRemindersItem->setLabel( i18nd("calendarsupport6", "Enable reminders for new Events") );
  mDefaultEventRemindersItem->setToolTip( i18nd("calendarsupport6", "By default, enable reminders for new events") );
  mDefaultEventRemindersItem->setWhatsThis( i18nd("calendarsupport6", "Check this box if you want to enable reminders for all newly created Events. You can always turn-off the reminders in the Event editor dialog.") );
  addItem( mDefaultEventRemindersItem, QStringLiteral( "defaultEventReminders" ) );
  mDefaultTodoRemindersItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Enable To-do Reminders" ), mDefaultTodoReminders, true );
  mDefaultTodoRemindersItem->setLabel( i18nd("calendarsupport6", "Enable reminders for new To-dos") );
  mDefaultTodoRemindersItem->setToolTip( i18nd("calendarsupport6", "By default, enable reminders for new to-dos") );
  mDefaultTodoRemindersItem->setWhatsThis( i18nd("calendarsupport6", "Check this box if you want to enable reminders for all newly created To-dos. You can always turn-off the reminders in the To-do editor dialog.") );
  addItem( mDefaultTodoRemindersItem, QStringLiteral( "defaultTodoReminders" ) );
  mReminderTimeItem = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "Default Reminder Time" ), mReminderTime, 15 );
  mReminderTimeItem->setLabel( i18nd("calendarsupport6", "Default reminder time") );
  mReminderTimeItem->setToolTip( i18nd("calendarsupport6", "Default time for reminders") );
  mReminderTimeItem->setWhatsThis( i18nd("calendarsupport6", "Enter the default reminder time for all newly created items. The time unit is specified in the adjacent combobox.") );
  addItem( mReminderTimeItem, QStringLiteral( "ReminderTime" ) );
  mReminderTimeUnitsItem = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "Default Reminder Time Units" ), mReminderTimeUnits, 0 );
  mReminderTimeUnitsItem->setLabel( i18nd("calendarsupport6", "Default Reminder Time Units") );
  mReminderTimeUnitsItem->setToolTip( i18nd("calendarsupport6", "Default time unit for reminders") );
  mReminderTimeUnitsItem->setWhatsThis( i18nd("calendarsupport6", "Enter the default reminder time units for all newly created items. The time is specified in the adjacent spinbox.") );
  addItem( mReminderTimeUnitsItem, QStringLiteral( "ReminderTimeUnits" ) );

  setCurrentGroup( QStringLiteral( "Time & Date" ) );

  mDayBeginsItem = new KConfigSkeleton::ItemDateTime( currentGroup(), QStringLiteral( "DayBegins" ), mDayBegins, QDateTime(QDate(1752,1,1), QTime(7,0)) );
  mDayBeginsItem->setLabel( i18nd("calendarsupport6", "Day begins at") );
  mDayBeginsItem->setWhatsThis( i18nd("calendarsupport6", "Enter the start time for events here. This time should be the earliest time that you use for events.") );
  addItem( mDayBeginsItem, QStringLiteral( "DayBegins" ) );
  mExcludeHolidaysItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Exclude Holidays" ), mExcludeHolidays, true );
  mExcludeHolidaysItem->setLabel( i18nd("calendarsupport6", "Exclude holidays") );
  mExcludeHolidaysItem->setWhatsThis( i18nd("calendarsupport6", "Check this box to prevent KOrganizer from marking the working hours on holidays.") );
  addItem( mExcludeHolidaysItem, QStringLiteral( "ExcludeHolidays" ) );
  mHolidaysItem = new KConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "Holidays" ), mHolidays );
  mHolidaysItem->setLabel( i18nd("calendarsupport6", "Use holiday regions:") );
  mHolidaysItem->setWhatsThis( i18nd("calendarsupport6", "Select which regions you want to use the holidays here. Defined holidays are shown as non-working days in the date navigator, the agenda view, etc.") );
  addItem( mHolidaysItem, QStringLiteral( "Holidays" ) );
  mHolidayCategoriesItem = new KConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "HolidayCategories" ), mHolidayCategories );
  mHolidayCategoriesItem->setLabel( i18nd("calendarsupport6", "Select holiday categories:") );
  mHolidayCategoriesItem->setToolTip( i18nd("calendarsupport6", "Select which holiday categories to display") );
  mHolidayCategoriesItem->setWhatsThis( i18nd("calendarsupport6", "Select which holiday categories to display (public, cultural, religious, financial).") );
  addItem( mHolidayCategoriesItem, QStringLiteral( "HolidayCategories" ) );

  setCurrentGroup( QStringLiteral( "Group Scheduling" ) );

  mUseGroupwareCommunicationItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Use Groupware Communication" ), mUseGroupwareCommunication, true );
  mUseGroupwareCommunicationItem->setLabel( i18nd("calendarsupport6", "Use Groupware communication") );
  mUseGroupwareCommunicationItem->setWhatsThis( i18nd("calendarsupport6", "Check this box to enable automatic generation of mails when creating, updating or deleting events (or to-dos) involving other attendees. You should check this box if you want to use the groupware functionality (e.g. Configuring Kontact as a KDE Kolab client).") );
  addItem( mUseGroupwareCommunicationItem, QStringLiteral( "UseGroupwareCommunication" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesSendPolicy;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("InvitationPolicySend");
    choice.label = i18nd("calendarsupport6", "Send mails without asking.");
    valuesSendPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("InvitationPolicyAsk");
    choice.label = i18nd("calendarsupport6", "Ask for every individual attendee what to do.");
    valuesSendPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("InvitationPolicyDontSend");
    choice.label = i18nd("calendarsupport6", "Do not send invitation emails at all (this can break group scheduling for iTip compliant clients).");
    valuesSendPolicy.append( choice );
  }
  mSendPolicyItem = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "Send Policy" ), mSendPolicy, valuesSendPolicy, InvitationPolicySend );
  mSendPolicyItem->setLabel( i18nd("calendarsupport6", "Default policy for invitations to other users:") );
  addItem( mSendPolicyItem, QStringLiteral( "SendPolicy" ) );
  mAdditionalMailsItem = new KConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "AdditionalMails" ), mAdditionalMails );
  mAdditionalMailsItem->setLabel( i18nd("calendarsupport6", "AdditionalMails") );
  addItem( mAdditionalMailsItem, QStringLiteral( "AdditionalMails" ) );

  setCurrentGroup( QStringLiteral( "Hidden Options" ) );

  mShowTimeZoneSelectorInIncidenceEditorItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowTimeZoneSelectorInIncidenceEditor" ), mShowTimeZoneSelectorInIncidenceEditor, true );
  mShowTimeZoneSelectorInIncidenceEditorItem->setLabel( i18nd("calendarsupport6", "Show timezone selectors in the event and todo editor dialog.") );
  addItem( mShowTimeZoneSelectorInIncidenceEditorItem, QStringLiteral( "ShowTimeZoneSelectorInIncidenceEditor" ) );

  setCurrentGroup( QStringLiteral( "Archive Dialog" ) );

  mAutoArchiveItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Auto Archive" ), mAutoArchive, false );
  mAutoArchiveItem->setLabel( i18nd("calendarsupport6", "Regularly archive events") );
  addItem( mAutoArchiveItem, QStringLiteral( "AutoArchive" ) );
  mExpiryTimeItem = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "Expiry Time" ), mExpiryTime, 1 );
  mExpiryTimeItem->setLabel( i18nd("calendarsupport6", "If auto-archiving is enabled, events older than this amount will be archived. The unit of this value is specified in another field.") );
  addItem( mExpiryTimeItem, QStringLiteral( "ExpiryTime" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesExpiryUnit;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("UnitDays");
    choice.label = i18nd("calendarsupport6", "In days");
    valuesExpiryUnit.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("UnitWeeks");
    choice.label = i18nd("calendarsupport6", "In weeks");
    valuesExpiryUnit.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("UnitMonths");
    choice.label = i18nd("calendarsupport6", "In months");
    valuesExpiryUnit.append( choice );
  }
  mExpiryUnitItem = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "Expiry Unit" ), mExpiryUnit, valuesExpiryUnit, UnitMonths );
  mExpiryUnitItem->setLabel( i18nd("calendarsupport6", "The unit in which the expiry time is expressed.") );
  addItem( mExpiryUnitItem, QStringLiteral( "ExpiryUnit" ) );
  mArchiveFileItem = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Archive File" ), mArchiveFile );
  mArchiveFileItem->setLabel( i18nd("calendarsupport6", "URL of the file where old events should be archived") );
  addItem( mArchiveFileItem, QStringLiteral( "ArchiveFile" ) );
  mArchiveEventsItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Archive Events" ), mArchiveEvents, true );
  mArchiveEventsItem->setLabel( i18nd("calendarsupport6", "Archive events") );
  addItem( mArchiveEventsItem, QStringLiteral( "ArchiveEvents" ) );
  mArchiveTodosItem = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Archive Todos" ), mArchiveTodos, true );
  mArchiveTodosItem->setLabel( i18nd("calendarsupport6", "Archive to-dos") );
  addItem( mArchiveTodosItem, QStringLiteral( "ArchiveTodos" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesArchiveAction;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("actionDelete");
    choice.label = i18nd("calendarsupport6", "Delete old events");
    valuesArchiveAction.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("actionArchive");
    choice.label = i18nd("calendarsupport6", "Archive old events to a separate file");
    valuesArchiveAction.append( choice );
  }
  mArchiveActionItem = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "Archive Action" ), mArchiveAction, valuesArchiveAction, actionArchive );
  mArchiveActionItem->setLabel( i18nd("calendarsupport6", "What to do when archiving") );
  addItem( mArchiveActionItem, QStringLiteral( "ArchiveAction" ) );

  setCurrentGroup( QStringLiteral( "Internal Settings" ) );

  mWorkWeekMaskItem = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "Work Week Mask" ), mWorkWeekMask, 31 );
  mWorkWeekMaskItem->setLabel( i18nd("calendarsupport6", "Work Week Mask") );
  addItem( mWorkWeekMaskItem, QStringLiteral( "WorkWeekMask" ) );
  mUserNameItem = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "user_name" ), mUserName, i18n("Anonymous") );
  mUserNameItem->setLabel( i18nd("calendarsupport6", "Full &name") );
  mUserNameItem->setWhatsThis( i18nd("calendarsupport6", "Enter your full name here. This name will be displayed as \"Organizer\" in to-dos and events you create.") );
  addItem( mUserNameItem, QStringLiteral( "UserName" ) );
  mUserEmailItem = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "user_email" ), mUserEmail, i18n("nobody@nowhere") );
  mUserEmailItem->setLabel( i18nd("calendarsupport6", "E&mail address") );
  mUserEmailItem->setWhatsThis( i18nd("calendarsupport6", "Enter here your e-mail address. This e-mail address will be used to identify the owner of the calendar, and displayed in events and to-dos you create.") );
  addItem( mUserEmailItem, QStringLiteral( "UserEmail" ) );
  mEventTemplatesItem = new KConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "EventTemplates" ), mEventTemplates );
  mEventTemplatesItem->setLabel( i18nd("calendarsupport6", "EventTemplates") );
  addItem( mEventTemplatesItem, QStringLiteral( "EventTemplates" ) );
  mTodoTemplatesItem = new KConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "TodoTemplates" ), mTodoTemplates );
  mTodoTemplatesItem->setLabel( i18nd("calendarsupport6", "TodoTemplates") );
  addItem( mTodoTemplatesItem, QStringLiteral( "TodoTemplates" ) );
  mJournalTemplatesItem = new KConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "JournalTemplates" ), mJournalTemplates );
  mJournalTemplatesItem->setLabel( i18nd("calendarsupport6", "JournalTemplates") );
  addItem( mJournalTemplatesItem, QStringLiteral( "JournalTemplates" ) );

  setCurrentGroup( QStringLiteral( "Colors" ) );

  mUnsetCategoryColorItem = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Unset Category Color" ), mUnsetCategoryColor, QColor( 151, 235, 121 ) );
  mUnsetCategoryColorItem->setLabel( i18nd("calendarsupport6", "\"No tag\" color (for \"Only tag\" drawing schemes)") );
  mUnsetCategoryColorItem->setToolTip( i18nd("calendarsupport6", "Use this color when drawing items without any tags") );
  mUnsetCategoryColorItem->setWhatsThis( i18nd("calendarsupport6", "Select a color to use for the \"no tag\" or \"unset tag\" situation, when an item does not have any tags.  This color is used when drawing items in the agenda or month views using the \"Only tag\" scheme.") );
  addItem( mUnsetCategoryColorItem, QStringLiteral( "UnsetCategoryColor" ) );
}

KCalPrefsBase::~KCalPrefsBase()
{
}

